@@ -33,13 +33,14 @@
 
 #include "cheese-thumb-view.h"
 
+#define THUMB_VIEW_MINIMUM_WIDTH 140
+#define THUMB_VIEW_MINIMUM_HEIGHT 100
 
 #define CHEESE_THUMB_VIEW_GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), CHEESE_TYPE_THUMB_VIEW, CheeseThumbViewPrivate))
 
 G_DEFINE_TYPE (CheeseThumbView, cheese_thumb_view, GTK_TYPE_ICON_VIEW);
 
-
 typedef struct
 {
   GtkListStore *store;
@@ -48,6 +49,7 @@ typedef struct
   GFileMonitor   *video_file_monitor;
   GnomeDesktopThumbnailFactory *factory;
   gboolean multiplex_thumbnail_generator;
+  guint n_items;
 } CheeseThumbViewPrivate;
 
 enum
@@ -84,15 +86,15 @@ cheese_thumb_view_thread_append_item (gpointer data)
   CheeseThumbViewPrivate    *priv       = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
 
   GnomeDesktopThumbnailFactory *factory = priv->factory;
-  GFile                 *file    = item->file;
-  GtkTreeIter            iter    = item->iter;
-  GdkPixbuf             *pixbuf  = NULL;
-  GFileInfo             *info;
-  char                  *thumb_loc;
-  GTimeVal               mtime;
-  char                  *mime_type;
-  char                  *uri;
-  char                  *filename;
+  GFile                        *file    = item->file;
+  GtkTreeIter                   iter    = item->iter;
+  GdkPixbuf                    *pixbuf  = NULL;
+  GFileInfo                    *info;
+  char                         *thumb_loc;
+  GTimeVal                      mtime;
+  char                         *mime_type;
+  char                         *uri;
+  char                         *filename;
 
   info = g_file_query_info (file, "standard::content-type,time::modified", 0, NULL, NULL);
 
@@ -180,7 +182,7 @@ cheese_thumb_view_append_item (CheeseThumbView *thumb_view, GFile *file)
   GtkTreePath  *path;
   char         *filename, *basename, *col_filename;
   GError       *error = NULL;
-  gboolean      skip = FALSE;
+  gboolean      skip  = FALSE;
 
   CheeseThumbViewThreadData *data;
 
@@ -194,7 +196,6 @@ cheese_thumb_view_append_item (CheeseThumbView *thumb_view, GFile *file)
 
   if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store), &iter))
   {
-
     /* check if the selected item is the first, else go through the store */
     gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter, THUMBNAIL_URL_COLUMN, &col_filename, -1);
     if (g_ascii_strcasecmp (col_filename, filename))
@@ -223,7 +224,7 @@ cheese_thumb_view_append_item (CheeseThumbView *thumb_view, GFile *file)
   {
     char *f;
 
-    f = g_strdup_printf ("%s/pixmaps/cheese-%i.svg", PACKAGE_DATADIR, g_random_int_range (1, 4));
+    f      = g_strdup_printf ("%s/pixmaps/cheese-%i.svg", PACKAGE_DATADIR, g_random_int_range (1, 4));
     pixbuf = gdk_pixbuf_new_from_file (f, NULL);
     g_free (f);
   }
@@ -394,8 +395,8 @@ cheese_thumb_view_on_drag_data_get_cb (GtkIconView      *thumb_view,
 
     g_free (str);
   }
-  gtk_selection_data_set (data, data->target, 8,
-                          (guchar *) uris, strlen (uris));
+  gtk_selection_data_set (data, gtk_selection_data_get_target (data),
+                          8, (guchar *) uris, strlen (uris));
   g_free (uris);
   g_list_free (list);
 }
@@ -537,9 +538,7 @@ cheese_thumb_view_fill (CheeseThumbView *thumb_view)
 static void
 cheese_thumb_view_finalize (GObject *object)
 {
-  CheeseThumbView *thumb_view;
-
-  thumb_view = CHEESE_THUMB_VIEW (object);
+  CheeseThumbView *thumb_view = CHEESE_THUMB_VIEW (object);
   CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
 
   g_object_unref (priv->store);
@@ -561,6 +560,32 @@ cheese_thumb_view_class_init (CheeseThumbViewClass *klass)
   g_type_class_add_private (klass, sizeof (CheeseThumbViewPrivate));
 }
 
+static void
+cheese_thumb_view_row_inserted_cb (GtkTreeModel *tree_model,
+                                   GtkTreePath  *path,
+                                   GtkTreeIter  *iter,
+                                   CheeseThumbView *thumb_view)
+{
+  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+
+  priv->n_items++;
+  gtk_widget_set_size_request (GTK_WIDGET (thumb_view), -1, -1);
+}
+
+static void
+cheese_thumb_view_row_deleted_cb (GtkTreeModel *tree_model,
+                                   GtkTreePath  *path,
+                                   CheeseThumbView *thumb_view)
+{
+  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+
+  priv->n_items--;
+  if (priv->n_items == 0)
+    gtk_widget_set_size_request (GTK_WIDGET (thumb_view),
+                                 THUMB_VIEW_MINIMUM_WIDTH,
+                                 THUMB_VIEW_MINIMUM_HEIGHT);
+}
+
 static void
 cheese_thumb_view_init (CheeseThumbView *thumb_view)
 {
@@ -569,17 +594,32 @@ cheese_thumb_view_init (CheeseThumbView *thumb_view)
   char *path_videos = NULL, *path_photos = NULL;
 
   GFile    *file;
-  const int THUMB_VIEW_HEIGHT = 120;
 
   eog_thumbnail_init ();
 
   priv->store = gtk_list_store_new (3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);
+  priv->n_items = 0;
+
+  g_signal_connect (G_OBJECT (priv->store),
+                    "row-inserted",
+                    G_CALLBACK (cheese_thumb_view_row_inserted_cb),
+                    thumb_view);
+  g_signal_connect (G_OBJECT (priv->store),
+                    "row-deleted",
+                    G_CALLBACK (cheese_thumb_view_row_deleted_cb),
+                    thumb_view);
 
   priv->fileutil = cheese_fileutil_new ();
 
   gtk_icon_view_set_model (GTK_ICON_VIEW (thumb_view), GTK_TREE_MODEL (priv->store));
 
-  gtk_widget_set_size_request (GTK_WIDGET (thumb_view), -1, THUMB_VIEW_HEIGHT);
+  gtk_widget_set_size_request (GTK_WIDGET (thumb_view),
+                               THUMB_VIEW_MINIMUM_WIDTH,
+                               THUMB_VIEW_MINIMUM_HEIGHT);
+
+  gtk_icon_view_set_margin (GTK_ICON_VIEW (thumb_view), 0);
+  gtk_icon_view_set_row_spacing (GTK_ICON_VIEW (thumb_view), 0);
+  gtk_icon_view_set_column_spacing (GTK_ICON_VIEW (thumb_view), 0);
 
   path_videos = cheese_fileutil_get_video_path (priv->fileutil);
   path_photos = cheese_fileutil_get_photo_path (priv->fileutil);
@@ -608,11 +648,8 @@ cheese_thumb_view_init (CheeseThumbView *thumb_view)
   }
 
   gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (thumb_view), 0);
-#ifdef HILDON
-  gtk_icon_view_set_columns (GTK_ICON_VIEW (thumb_view), -1);
-#else
+
   gtk_icon_view_set_columns (GTK_ICON_VIEW (thumb_view), G_MAXINT);
-#endif
 
   gtk_icon_view_enable_model_drag_source (GTK_ICON_VIEW (thumb_view), GDK_BUTTON1_MASK,
                                           target_table, G_N_ELEMENTS (target_table),
